<?php

//
// innovaphone fax mail forward PHP script
// copyright (c) innovaphone 2014
//

$path=dirname(__FILE__);
require_once("$path/../include.php");
require_once("classes/translator.class.php");
require_once("$path/../app-classes/innovaphone.fax.config.class.php");
require_once("$path/../app-classes/innovaphone.fax.shm.class.php");
require_once("$path/../app-classes/innovaphone.fax.database.class.php");
require_once("classes/mail/innovaphone.mailbox.class.php");

$version = file_get_contents("$path/../version");

declare(ticks = 1);
?>

innovaphone fax mail forward PHP script
copyright (c) innovaphone 2014
<?php

print "Version " . $version . "\n\n";

$help = false;
foreach($argv as $arg) {
    if(in_array($arg, array('--help', '-help', '-h', '-?'))) {
        $help = true;
    }
}

if(!is_array($_GET) && ($argc < 3 || $help)) {
?>
    Usage:
  <?php echo $argv[0]; ?> <id> <jobGuid>

<id> is the configuration database id specific for this server instance.
<jobGuid> is the guid of the fax job which should be handled.

With the --help, -help, -h, or -? options, you can get this help.

<?php
exit(1);
}

$configurationId        = null;
$jobGuid                = null;

if($argc >= 2) {
    $configurationId    = intval($argv[1]);
    $jobGuid            = $argv[2];
}
else if(is_array($_GET)) {
    $configurationId    = intval($_GET['id']);
    $jobGuid            = $_GET['jobGuid'];
}


setlocale(LC_CTYPE, "en_US.UTF-8");

function getLogDate() {
    $now = new DateTime();
    return date('Ymd-His') . "  -  ";
}

function sig_handler($nsig)
{
    print "\n";
    print getLogDate() . "terminated($nsig).\n";
    exit;
}

pcntl_signal(SIGTERM, "sig_handler", false);
pcntl_signal(SIGINT, "sig_handler", false);
pcntl_signal(SIGHUP, "sig_handler", false);

$faxShm     = null;


function shutdown_function() {
    global $faxShm;
    global $jobGuid;
    if($faxShm && $jobGuid) {
        $faxShm->unlockJob($jobGuid);
    }
    print "\n";
}

register_shutdown_function('shutdown_function');


function generateMessageID($domain = "") {
    $t = explode(" ", microtime());
    $localPart  = date("YmdHis", $t[1]);
    $localPart .= ".";
    $localPart .= base_convert(substr((string)$t[0], 2, 5), 10, 36);
    $localPart .= base_convert(bin2hex(openssl_random_pseudo_bytes(2)), 16, 36);
    $localPart .= base_convert(getmypid(), 10, 36);
    $domainPart = gethostname();
    if($domainPart != "" && $domain != "") {
        $domainPart .= ".";
    }
    if($domain != "") {
        $domainPart .= $domain;
    }
    return "<" . strtoupper($localPart) . "@" . $domainPart . ">";
}


print getLogDate() . "Reading configuration... ";
$cfg     = new FaxServer();
$cfg->read("", $configurationId);
if(!$cfg) {
    print "failed.\n";
    die("Could not read configuration.");
}
print "done.\n";


tl::change("en");
tl::register("faxserver", dirname(__FILE__) . "/../lang");
tl::change($cfg->lang);


print getLogDate() . "Attaching to shared memory... ";
$faxShm     = new InnovaphoneFaxShm($cfg->shmId);
if(!$faxShm->isAttached()) {
    print "failed.\n";
    die("Could not attach to the SHM.");
}
print "done.\n";


print getLogDate() . "Trying to lock job... ";
$ret = $faxShm->lockJob($jobGuid, null, null);
if(!$ret) {
    $faxShm = 0;
    print "failed.\n";
    die("Could not lock fax job.");
}
print "done.\n";


print getLogDate() . "Connecting to mailbox... ";
$mailbox    = new InnovaphoneMailbox($cfg->mailHost, $cfg->mailPort, $cfg->mailUser, $cfg->mailPassword, $cfg->mailFolder);
if(!$mailbox->isConnected()) {
    print "failed.\n";
    die("Could not connect to the mailbox.");
}
print "done.\n";


print getLogDate() . "Connecting to database... ";
$faxDatabase    = new InnovaphoneFaxDatabase();
if(!$faxDatabase->isConnected()) {
    print "failed.\n";
    die("Could not connect to the database.");
}
$faxDatabase->setServerId($cfg->id);
print "done.\n";


print getLogDate() . "Reading fax job database entry... ";
$faxJob     = $faxDatabase->getFaxJob($jobGuid);
if(!$faxJob) {
    print "failed.\n";
    die("Could not read database object.");   
}
print "done.\n";


if($faxJob && $faxJob->getDirection() != 'incoming') {
    die("Fax job with wrong direction found.");
}

if($faxJob->getProgress() == 'converted')
{
    print getLogDate() . "Forwarding received fax from '" . $faxJob->getRemoteNumber() . "' to '" . $faxJob->getUserH323() . "'.\n";
    
    $fileName           = $faxJob->getFileName();
    $fileNameParts      = explode('.', $fileName);
    $fileNameExtention  = $fileNameParts[count($fileNameParts) - 1];
    $documents          = array();
    $documents[]        = $faxJob->getFileName();
    $pages              = 0;
    $maxPages           = 30;
    $messageId          = "";
    
    if(!file_exists($fileName)) {
        print getLogDate() . "File doesn't exist.\n";
    }
    else {
    
        $exec           = "";
    
        if($fileNameExtention == "pdf") {
            $exec   = 'gs -q -dNODISPLAY -c "(' . escapeshellcmd($faxJob->getFileName()) . ') (r) file runpdfbegin pdfpagecount = quit"';
        }
        else if($fileNameExtention == "tif") {
            $exec   = 'tiffinfo ' . escapeshellcmd($faxJob->getFileName()) . ' | grep "Page Number" | grep -c "P"';
        }

        if($exec != "") {
            $outputs    = array();
            $ret        = false;
            exec($exec, $outputs, $ret);

            if(!$ret && count($outputs)) {
                $pages = intval($outputs[0]);
            }
        }
    
        if($pages > $maxPages) {
            $documents          = array();

            print getLogDate() . "Splitting file '" . $fileName . "' with " . $pages . " pages... ";

            for($i = 1; $i <= $pages; $i = $i + $maxPages) {
                $fileNameSplit      = substr_replace($fileName, "-" . floor($i/$maxPages), - strlen($fileNameExtention) - 1, 0);
                $exec               = "";
                if($fileNameExtention == "pdf") {
                    $exec = 'gs -q -dNOPAUSE -dBATCH -sOutputFile="' . $fileNameSplit . '" -dFirstPage=' . $i . ' -dLastPage=' . ($i + $maxPages - 1) . ' -sDEVICE=pdfwrite "' . $fileName .'"';
                }
                else if($fileNameExtention == "tif") {
                    $exec = "tiffcrop -N " . $i . "-" . ($i + $maxPages - 1) . " " . escapeshellarg($fileName) . " " . escapeshellarg($fileNameSplit);
                }
                if($exec != "") {
                    $outputs    = array();
                    $ret        = true;
                    exec($exec, $outputs, $ret);
            
                    if($ret) {
                        print "failed.\n";
                        $output = "";
                        foreach($outputs as &$row) {
                            $output .= $row . "\n";
                        }

                        print "Command: \n" . $exec . "\n";
                        print "Output: \n" . $output . "\n";
                        die("Failed splitting file.");
                    }
                    $documents[] = $fileNameSplit;
                }
            }
        
            print "done.\n";
        }
    
        foreach($documents as $count => $fileName) {
        
            print getLogDate() . "Saving message with $fileName... ";

            $messageId = generateMessageID($cfg->faxServerDomain);
                
            setlocale(LC_CTYPE, "en_US.UTF-8");
            if($faxJob->getUserLanguage() != "") {
                tl::change($faxJob->getUserLanguage());
            }
                
            $number                         = $faxJob->getRemoteNumber() != "" ? $faxJob->getRemoteNumber() : "unknown";
            $to                             = '"' . mb_encode_mimeheader($faxJob->getUserName(), "UTF-8", "Q", "") . '" <' . $faxJob->getUserH323() . '>';
            $from                           = imap_rfc822_write_address("faxserver", $cfg->faxServerDomain, $cfg->name);
            $replyTo                        = imap_rfc822_write_address($number,
                $cfg->faxServerDomain,
                $faxJob->getRemoteNumber() != "" ? $faxJob->getRemoteNumber() : tl::tl("Unknown fax number"));
            $subject                        = $cfg->getString(FaxServerStrings::msgSubjectIncomingSuccess, $faxJob->getUserLanguage());
            $message                        = $cfg->getString(FaxServerStrings::msgIncomingSuccess, $faxJob->getUserLanguage());
            $fileDate                       = $faxJob->getCallTimestamp() ? $faxJob->getCallTimestamp()->format("-Ymd-Hi") : "";
            $fileDate                       .= $count != 0 ? ("-" . $count) : "";
            $fileNameParts                  = explode('.', $fileName);
            $fileNameExtention              = $fileNameParts[count($fileNameParts) - 1];
            $files                          = array("fax-$number$fileDate.$fileNameExtention" => $fileName);
            $headerFileName                 = substr($fileName, 0, -strlen($fileNameExtention)) . "txt";
            $messageFileName                = substr($fileName, 0, -strlen($fileNameExtention)) . "msg";

            $info                           = array();
            $info['<result>']               = $faxJob->getResult();
            $info['<user-name>']            = $faxJob->getUserName();
            $info['<user-h323>']            = $faxJob->getUserH323();
            $info['<user-number>']          = $faxJob->getUserNumber();
            $info['<remote-number>']        = $faxJob->getRemoteNumber() != "" ? $faxJob->getRemoteNumber() : tl::tl("unknown");
            $info['<remote-subscriber-id>'] = $faxJob->getRemoteSubscriberId();
            $info['<file-name>']            = $fileName;
            $info['<calls>']                = $faxJob->getCalls();
            $info['<pages>']                = $faxJob->getPages();
            $info['<duration>']             = $faxJob->getDuration() ? $faxJob->getDuration()->format('%i:%S') : "-";
            $info['<error-cause>']          = $faxJob->getErrorCause();
            $info['<error-description>']    = $faxJob->getErrorDescription();
            $info['<date>']                 = $faxJob->getCallTimestamp() ? InputPageDateField::translateDate(($faxJob->getCallTimestamp()->getTimestamp())) : "-";
            $info['<time>']                 = $faxJob->getCallTimestamp() ? $faxJob->getCallTimestamp()->format("H:i:s") : "-";

            $message = strtr($message, $info);
            if($faxJob->getRemoteSubscriberId() == "") {
                $info['<remote-subscriber-id>'] = $info['<remote-number>'];
            }
            $subject = strtr($subject, $info);

            $headerLines = InnovaphoneMailbox::saveMail($messageFileName, $to, $from, $subject, $message, $files);

            $header = "";
            foreach($headerLines as $headerType => &$headerLine) {
                $header .= $headerType . ": " . $headerLine . "\r\n";
            }
            
            $header .= "Envelope-ID: <" . $faxJob->getGuid() . ">\r\n";
            $header .= "Message-ID: " . $messageId . "\r\n";
            if($faxJob->getRemoteNumber() != "") {
                $header .= "Reply-To: " . $replyTo . "\r\n";
            }
            $header .="\r\n";
    
            $headerFile = fopen($headerFileName, 'w');
            $ret = fwrite($headerFile, $header);
            if($ret === false) {
                print "failed.\n";
                die("Couldn't write header file.");
            }
            fclose($headerFile);

            print "done.\n";


            print getLogDate() . "Sending mail with message id " . $messageId . " to '" . $faxJob->getUserH323() . "'... ";
    
            $headerFileName  = escapeshellarg($headerFileName);
            $messageFileName = escapeshellarg($messageFileName);
    
            $exec       = "cat $headerFileName $messageFileName | /usr/sbin/sendmail -t -i -f return-mail@" . $cfg->faxServerDomain . " -V \"<" . $faxJob->getGuid() . ">\" 2>&1";
            $outputs    = array();
            $ret        = false;

            exec($exec, $outputs, $ret);

            if($ret) {
                print "failed.\n";
                $output = "";
                foreach($outputs as &$row) {
                    $output .= $row . "\n";
                }
                print $output;
                die("Sending mail failed.");
            }
            else {
                print "done.\n";
            }
        
            print getLogDate() . "Deleting temporary files... ";
            $files      = $headerFileName . " " . $messageFileName;
            if($faxJob->getFileName() != $fileName) {
                $files .= " " . escapeshellarg($fileName);
            }
            $exec       = "rm -f $files 2>&1";
            $outputs    = array();
            $ret        = false;
            exec($exec, $outputs, $ret);
            if($ret) {
                print "failed.\n";
            }
            else {
                print "done.\n";
            }
        }
    }
    
    print getLogDate() . "Updating database... ";
    $faxJob->setProgress('closed');
    $faxJob->setResult('success');
    $faxJob->setMessageId($messageId);
    $faxJob = $faxDatabase->updateFaxJob($faxJob);
    if(!$faxJob || $faxJob->getProgress() != 'closed') {
        print "failed.\n";
    }
    else {
        print "done.\n";

        print getLogDate() . "Deleting received document... ";
        $files      = " " . escapeshellarg($faxJob->getFileName());
        $exec       = "rm -f $files 2>&1";
        $outputs    = array();
        $ret        = false;
        exec($exec, $outputs, $ret);
        if($ret) {
            print "failed.\n";
        }
        else {
            print "done.\n";
        }
    }
}

print getLogDate() . "Finished.\n";
?>
